# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect


# Unused Cached Class
class CachedProperty(object):

    def __init__(self, func):
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        if "__cached__" not in obj.__dict__:
            obj.__dict__["__cached__"] = []
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        obj.__dict__["__cached__"].append(self.func.__name__)
        return value

    @property
    def annotation(self):
        return inspect.signature(self.func).return_annotation


def clear_cache(obj):
    if "__cached__" not in obj.__dict__:
        return
    for cached_key in obj.__dict__["__cached__"]:
        obj.__dict__.pop(cached_key)
    obj.__dict__["__cached__"] = []